;
;      SCRIPT: 1MM.SC
;      AUTHOR: Matthew Hopkins
; DESCRIPTION: This script contains PAL code demonstrating 1MM relationship
;              dialog box pulled from tables (Level1/Level2/Level3)

PROC Init.u()
  PRIVATE i,element.r,fieldlist.r
  FOR i FROM 1 TO ARRAYSIZE(TableList.r)
    MESSAGE "Loading dynamic array from table "+ExtractFileName.a(TableList.r[i])
    IF (i=1) THEN
      EXECUTE "DYNARRAY "+ExtractFileName.a(TableList.r[i])+".y[]"
    ENDIF
    VIEW ExtractFileName.a(TableList.r[i])
    ; Need to create a DYNARRAY for every record - if the field value is
    ; numeric, then we convert it to an alpha (i.e. 1=a 2=b ..) and then
    ; place the expression !C! in the element name. For example, a field
    ; might be contain the value - 13.  This would translate into the
    ; dynamic array called ac.!c!.y[]. If this is a secondary table
    ; where there are two key fields with the values [A01][12], then this
    ; would be converted into the array name of A01..ac.!c!.y[]  If a space
    ; exists in an alphanumeric field, then we need to convert the space to
    ; double underscore (i.e. __).  For example, "MY FIELD" would be converted
    ; to the DYNARRAY names my__field.y[].
    SCAN
      ARRAY element.r[i]
      ARRAY fieldlist.r[i]
      FOR j FROM 1 TO ARRAYSIZE(element.r)
        fieldlist.r[j] = ExtractFieldNumber.n(TableList.r[j],j)
        IF (fieldlist.r[j] = 0) THEN
          RETURN FALSE
        ENDIF
      ENDFOR
      IF (i=1) THEN
        group.a = ExtractFileName.a(TableList.r[i]) + "."
      ELSE
        group.a = ""
        FOR j FROM 1 TO ARRAYSIZE(fieldlist.r)
          IF (j<>ARRAYSIZE(fieldlist.r)) THEN
            CTRLHOME
            WHILE (FIELDNO(FIELD(),TABLE())<>fieldlist.r[j])
              RIGHT
            ENDWHILE
            group.a = group.a + FieldConvert.a([])
          ENDIF
        ENDFOR
      ENDIF
      element.a = ""
      FOR j FROM 1 TO ARRAYSIZE(fieldlist.r)
        CTRLHOME
        WHILE (FIELDNO(FIELD(),TABLE())<>fieldlist.r[j])
          RIGHT
        ENDWHILE
        element.a = element.a + FieldConvert.a([])
      ENDFOR
      EXECUTE "DYNARRAY "+element.a+"y[]"
      EXECUTE group.a+"y[\""+element.a+"y\"]=["+ExtractDescription.a(TableList.r[i])+"]"
    ENDSCAN
  ENDFOR
ENDPROC

PROC ExtractFileName.a(filename.a)
  PRIVATE dummy.a,dumdum.a
  IF (SEARCH("\\",filename.a)=0) THEN
    retval = MATCH(filename.a,"..::..::..",filename.a,dummy.a,dumdum.a)
    RETURN filename.a
  ELSE
    DEBUG ; I'll write this section later
  ENDIF
ENDPROC

PROC ExtractFieldNumber.n(string.a,number.n)
  PRIVATE dummy.a,fieldlist.a,nslashes.n,where.n,dumdum.a,i,ntotalfields.n
  retval = MATCH(string.a,"..::..::..",dummy.a,fieldlist.a,dumdum.a)
  IF retval THEN
    ; Get the nth number in the list seperated by slashes (i.e. 1/2/3)
    ; so.. first we'll find out how many numbers are in the list of fields.
    nslashes.n = 0
    IF (SEARCH("/",fieldlist.a)<>0) THEN
      where.n = SEARCH("/",fieldlist.a)
      nslashes.n = nslashes.n + 1
      WHILE TRUE
        where.n = SEARCHFROM("/",fieldlist.a,where.n+1)
        IF (where.n<>0) THEN
          nslashes.n = nslashes.n + 1
        ELSE
          QUITLOOP
        ENDIF
      ENDWHILE
      IF (number.n=1) THEN
        RETURN NUMVAL(SUBSTR(fieldlist.a,1,1))
      ELSE
        where.n = SEARCH("/",fieldlist.a)
        FOR i FROM 1 TO MAX((number.n-2),0)
          where.n = SEARCHFROM("/",fieldlist.a,where.n+1)
        ENDFOR
        RETURN NUMVAL(SUBSTR(fieldlist.a,where.n+1,1))
      ENDIF
    ELSE
      RETURN 1
    ENDIF
  ELSE
    RETURN 0
  ENDIF
ENDPROC

PROC ExtractDescription.a(string.a)
  PRIVATE dummy.a,dumdum.a,description.a
  retval = MATCH(string.a,"..::..::..",dummy.a,dumdum.a,description.a)
  IF retval THEN
    RETURN description.a
  ELSE
    RETURN FALSE
  ENDIF
ENDPROC

PROC FieldConvert.a(string.v)
  PRIVATE i,returnstr.a
  SWITCH
    CASE (SEARCH(TYPE(string.v),"NS$")<>0) :
      returnstr.a = ""
      FOR i FROM 1 TO LEN(string.v)
        returnstr.a = returnstr.a + CHR(NUMVAL(SUBSTR(string.v,i,1))+ASC("@"))
      ENDFOR
      returnstr.a = returnstr.a + "."
    CASE (TYPE(string.v) = "D")           :
    OTHERWISE                             :
      returnstr.a = AddSlashes.a(string.v) + "."
  ENDSWITCH
  RETURN returnstr.a
ENDPROC

PROC AddSlashes.a(string.a)
  PRIVATE part1.a,part2.a
  retval = MATCH(string.a,".. ..",part1.a,part2.a)
  IF retval THEN
    WHILE retval
      retval = MATCH(string.a,".. ..",part1.a,part2.a)
      string.a = part1.a + "__" + part2.a
    ENDWHILE
  ENDIF
  RETURN string.a
ENDPROC

PROC GenerateDisplayArray.u(ArrayName.a,currentarray.a)
  executestring.a = "DYNARRAY "+ArrayName.a+".y[] "+
                    "FOREACH element.a IN "+CurrentArray.a+" "+
                    "  "+ArrayName.a+".y[element.a] = "+CurrentArray.a+"[element.a] "+
                    "ENDFOREACH"
  IF (LEN(executestring.a)<=255) THEN
    EXECUTE executestring.a
  ELSE
    FILEWRITE "$BRANCH$.SC" FROM executestring.a
    PLAY "$BRANCH$"
  ENDIF
ENDPROC

PROC Dialog.u()
  MESSAGE "Loading Dialog"
  SHOWDIALOG "Groceries"
    PROC "DialogControl.u"
    TRIGGER "SELECT","OPEN"
    @3,22 HEIGHT 19 WIDTH 33

    PICKDYNARRAY @1,1 HEIGHT 3 WIDTH 27
      COLUMNS 1
      TopArray.y
      TAG "TopArray"
      TO TopArray.a

    PICKDYNARRAY @5,1 HEIGHT 4 WIDTH 27
      COLUMNS 1
      MiddleArray.y
      TAG "MiddleArray"
      TO MiddleArray.a

    PICKDYNARRAY @10,1 HEIGHT 4 WIDTH 27
      COLUMNS 1
      BottomArray.y
      TAG "BottomArray"
      TO BottomArray.a

    PUSHBUTTON @15,7 WIDTH 6
      "Ok"
      OK
      VALUE "OK"
      TAG "OKButton"
      TO ButtonValue.a

    PUSHBUTTON @15,13 WIDTH 10
      "Cancel"
      CANCEL
      VALUE "Cancel"
      TAG "CancelButton"
      TO ButtonValue.a
 ENDDIALOG
ENDPROC

PROC DialogControl.u(EventType,TagValue,EventValue,ElementValue)
  PRIVATE where.n
  SWITCH
    CASE (EventType = "OPEN") :
      MESSAGE ""
    CASE (EventType = "SELECT") :
      SWITCH
        CASE (TagValue = "TopArray") :
          MiddleArrayName.a = CONTROLVALUE(TagValue)
          GenerateDisplayArray.u("MiddleArray",MiddleArrayName.a)
          REFRESHDIALOG
          BottomArrayName.a = MiddleArray.a
          GenerateDisplayArray.u("BottomArray",BottomArrayName.a)
          REFRESHDIALOG
        CASE (TagValue = "MiddleArray") :
          BottomArrayName.a = CONTROLVALUE(TagValue)
          RefreshArrays.u()
          REFRESHDIALOG
      ENDSWITCH
  ENDSWITCH
ENDPROC

PROC RefreshArrays.u()
  GenerateDisplayArray.u("TopArray",TopArrayName.a)
  GenerateDisplayArray.u("MiddleArray",MiddleArrayName.a)
  GenerateDisplayArray.u("BottomArray",BottomArrayName.a)
ENDPROC

RELEASE VARS ALL
ARRAY TableList.r[3]
TableList.r[1] = "Level1::1::Description"
TableList.r[2] = "Level2::1/2::Description"
TableList.r[3] = "Level3::1/2/3::Description"
DYNARRAY TopArray.y[]
DYNARRAY MiddleArray.y[]
DYNARRAY BottomArray.y[]
TopArrayName.a    = "Level1.y"
MiddleArrayName.a = "Canned.y"
BottomArrayName.a = "Canned.Soup.y"
Init.u()
CLEARALL

RefreshArrays.u()
Dialog.u()

SAVEVARS ALL
